# 带字符串的 Java `switch case`

> 原文： [https://howtodoinjava.com/java7/strings-in-switch-statement/](https://howtodoinjava.com/java7/strings-in-switch-statement/)

**`Switch`语句**也早于 Java 7，但它仅支持`int`和`enum`类型。 在 Java 7 发布之后，`Switch`语句也支持[**字符串**](https://howtodoinjava.com/java-string/)类。

## 1\. 带字符串的 Java `switch case`

在`switch case`语句中使用**字符串类的 Java 程序**。

```java
public class StringSupportedInSwitch 
{
	public static void main(String[] args)
	{
		System.out.println(getExpendedMessage("one"));
		System.out.println(getExpendedMessage("three"));
		System.out.println(getExpendedMessage("five"));
	}

	static String getExpendedMessage(final String token) 
    {
        String value = null;

        switch (token) 
        {
            case ("one"):
                value = "Token one identified";
                break;

            case ("two"):
                value = "Token two identified";
                break;

            case ("three"):
                value = "Token three identified";
                break;

            case ("four"):
                value = "Token four identified";
                break;

            default:
                value = "No token was identified";
       }

        return value;
    }
}

```

程序输出。

```java
Token one identified
Token three identified
No token was identified

```

## 2\. Java `switch case`处理多个条件

有时，我们想对`switch`语句中的多个`case`执行某些操作。 在这种情况下，我们可以在单独的情况下写入每个值，并且只有在所有情况都写完之后，才写下应用程序逻辑。

例如，在给定程序中，所有奇数标记将由第一个切换条件处理，偶数标记将由第二个切换条件处理。

具有多个条件的`switch case`的 Java 示例。

```java
public class StringSupportedInSwitch {

	public static void main(String[] args) 
	{
		System.out.println(getExpendedMessage("one"));
		System.out.println(getExpendedMessage("two"));
	}

	static String getExpendedMessage(final String token) 
	{
		String value = null;

		switch (token) 
		{
			case ("one"):
			case ("three"):
				value = "Odd token identified";
				break;

			case ("two"):
			case ("four"):
				value = "Even token identified";
				break;

			default:
				value = "No token was identified";
		}

		return value;
	}
}

```

程序输出：

```java
Odd token identified
Even token identified

```

在此示例中，我们学习了将 **Java `switch`语句与字符串**结合使用。 [Java 7 中引入了此特性](https://howtodoinjava.com/java7/java-7-changes-features-and-enhancements/)。

学习愉快！