# JDBC SQL `DELETE`查询示例

> 原文： [https://howtodoinjava.com/java/jdbc/jdbc-sql-delete-query-example/](https://howtodoinjava.com/java/jdbc/jdbc-sql-delete-query-example/)

在以前的文章中，我们了解了 [**JDBC 驱动的类型**](//howtodoinjava.com/java/jdbc/jdbc-basics-types-of-jdbc-drivers/ "JDBC Basics : Types of JDBC Drivers?")和[**如何使用 JDBC**](//howtodoinjava.com/java/jdbc/jdbc-mysql-database-connection-example/ "JDBC MySQL Database Connection Example")，然后是[建立数据库连接后**如何执行`SELECT`查询**](//howtodoinjava.com/misc/jdbc-select-query-example/ "JDBC SELECT Query Example")，然后是[**`INSET`查询示例**](//howtodoinjava.com/java/jdbc/jdbc-sql-insert-query-example/ "JDBC SQL INSERT Query Example") 。 让我们前进。 在此示例中，我将使用 JDBC 来执行 SQL `DELETE`查询。

![JDBC-Icon](img/353e2fc90002c7f65b66549c16f491fa.png)

执行 SQL `DELETE`查询以删除/删除关系数据库中存储的数据。 它需要执行以下步骤：

1）建立数据库连接
2）执行 SQL `DELETE`查询

先决条件包括设置数据库架构和首先创建表。

```java
CREATE SCHEMA 'JDBCDemo' ;

CREATE TABLE 'JDBCDemo'.'EMPLOYEE'
(
	'ID' INT NOT NULL DEFAULT 0 ,
	'FIRST_NAME' VARCHAR(100) NOT NULL ,
	'LAST_NAME' VARCHAR(100) NULL ,
	'STAT_CD' TINYINT NOT NULL DEFAULT 0
);

```

让我们在代码中编写以上步骤：

## 1）建立数据库连接

尽管我们已经在建立 JDBC 连接中学习了它，但是让我们用这个简单的代码片段来回顾一下。

```java
Class.forName("com.mysql.jdbc.Driver");
connection = DriverManager
	.getConnection("jdbc:mysql://localhost:3306/JDBCDemo", "root", "password");

```

## 2）执行 SQL `DELETE`查询

这是帖子中的主要步骤和核心部分。 它需要先创建一个`Statement`对象，然后使用它的`execute()`方法。

```java
Statement stmt = connection.createStatement();
stmt.execute("DELETE FROM EMPLOYEE WHERE ID >= 1");

```

上面的语句将在我们连接的数据库中执行`delete`语句。 这将删除所有与`where`子句匹配的记录。

让我们看看整个代码在工作。

```java
package com.howtodoinjava.jdbc.demo;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

public class DeleteDataDemo {
	public static void main(String[] args) {
		Connection connection = null;
		Statement stmt = null;
		try 
		{
			Class.forName("com.mysql.jdbc.Driver");
			connection = DriverManager.getConnection("jdbc:mysql://localhost:3306/JDBCDemo", "root", "password");

			stmt = connection.createStatement();
			stmt.execute("DELETE FROM EMPLOYEE WHERE ID >= 1");
		} 
		catch (Exception e) {
			e.printStackTrace();
		}finally {
			try {	
				stmt.close();
				connection.close();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}	
}

```

以上就是这篇文章。 如果需要解释，请给我评论。

**快乐学习！**