# Spring MVC JSTL 配置示例

> 原文： [https://howtodoinjava.com/spring-mvc/how-to-add-jstl-support-in-spring-3-using-maven/](https://howtodoinjava.com/spring-mvc/how-to-add-jstl-support-in-spring-3-using-maven/)

学习使用 maven 构建工具为 Spring MVC 应用配置 JSTL 支持。 学习在 [Spring MVC](https://howtodoinjava.com/spring-mvc-tutorial/) 应用中启用 **JSTL 标签**。

## 1\. JSTL maven 依赖项

```java
<dependency>
	<groupid>javax.servlet</groupid>
	<artifactid>jstl</artifactid>
	<version>1.2</version>
	<scope>runtime</scope>
</dependency>

<dependency>
	<groupid>taglibs</groupid>
	<artifactid>standard</artifactid>
	<version>1.1.2</version>
	<scope>runtime</scope>
</dependency>

```

## 2\. 配置`InternalResourceViewResolver`来解析 JSTL 视图

#### 2.1 Spring JSTL Java 配置

```java
@Bean
public ViewResolver configureViewResolver() 
{
	InternalResourceViewResolver viewResolve = new InternalResourceViewResolver();
	viewResolve.setPrefix("/WEB-INF/jsp/");
	viewResolve.setSuffix(".jsp");
	return viewResolve;
}

```

#### 2.2 Spring JSTL XML 配置

```java
<bean class="org.springframework.web.servlet.view.InternalResourceViewResolver">
	<property name="viewClass" value="org.springframework.web.servlet.view.JstlView"></property>
	<property name="prefix">
		<value>/WEB-INF/jsp/</value>
	</property>
	<property name="suffix">
		<value>.jsp</value>
	</property>
</bean>

```

## 3\. 在 JSP 文件中使用 JSTL 标记

```java
<%@ taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %> 

<h1>Welcome message : <c:out value="${message}"></c:out></h1>

```

学习愉快！

阅读更多：

[JSTL 库](https://www.oracle.com/technetwork/java/index-jsp-135995.html)