# Log4j Maven 配置示例

> 原文： [https://howtodoinjava.com/log4j/how-to-configure-log4j-using-maven/](https://howtodoinjava.com/log4j/how-to-configure-log4j-using-maven/)

[**Log4j**](https://logging.apache.org/log4j/1.2/ "log4j home page") 是一个简单而灵活的日志记录框架。 日志记录为开发人员提供了有关应用故障的详细上下文。 使用 log4j 可以在运行时启用日志记录，而无需修改应用二进制文件。 log4j 包的设计目的是使这些语句可以保留在出厂代码中，而不会造成高昂的性能成本。

以下是在项目中配置 log4j 日志记录支持的最基本步骤。

**1）创建一个 Maven 项目**

```java
mvn archetype:generate -DgroupId=com.howtodoinjava -DartifactId=Log4jTestProject
 -DarchetypeArtifactId=maven-archetype-quickstart -DinteractiveMode=false
```

在 Eclipse 工作区或您正在使用的任何其他 IDE 中运行上述命令。如果您的工作区中已经有一个项目，则直接转到步骤 3。

**2）将项目转换为 Eclipse 支持的 Java 项目**

```java
mvn eclipse:eclipse
```

上面的命令会将 maven 项目转换为 Eclipse Java 项目。 现在，将项目导入到 eclipse 中。

**3）使用 log4j 依赖项更新`pom.xml`文件**

在给定的依赖项下面添加到`pom.xml`。

```java
<dependency>
	<groupId>log4j</groupId>
	<artifactId>log4j</artifactId>
	<version>1.2.17</version>
</dependency>

```

运行以下命令以下载本地系统中所需的 jar，并更新项目运行时依赖项。

```java
mvn eclipse:eclipse
```

**4）使用`BasicConfigurator`测试应用**

```java
package com.howtodoinjava;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

public class Log4jHelloWorld {

	static final Logger logger = Logger.getLogger(Log4jHelloWorld.class);

	public static void main(String[] args)
	{
		//Configure logger
		BasicConfigurator.configure();
		logger.debug("Hello World!");
	}
}

```

**输出：**

```java
0 [main] DEBUG com.howtodoinjava.Log4jHelloWorld  - Hello World!
```

如果您在控制台中看到上述消息，那么恭喜您，我们已经成功配置了 log4j。 如果您有任何问题，请再次重复上述所有步骤，或给我留言。

**祝您学习愉快！**