# Gson – JSON 输出的精美打印

> 原文： [https://howtodoinjava.com/gson/pretty-print-json-output/](https://howtodoinjava.com/gson/pretty-print-json-output/)

默认情况下，Gson 以紧凑格式打印 JSON，即字段名称及其值，对象字段以及 JSON 输出中数组内的对象等之间将没有空格。

例如：

```java
{"id":1,"firstName":"Lokesh","lastName":"Gupta", "emailId":"howtogoinjava@gmail.com"}

```

要启用 **Gson 精美打印**功能，我们必须使用[`GsonBuilder`](https://howtodoinjava.com/gson/gson-gsonbuilder-configuration/)配置`Gson`实例。

默认情况下，Gson *格式化输出 JSON*，其默认行长为 80 个字符，2 个字符的缩进和 4 个字符的右边距。

## 1.精美打印 JSON 的语法

```java
Gson gson = new GsonBuilder()
				.setPrettyPrinting()
				.create();

String jsonOutput = gson.toJson(someObject);

```

## 2\. Gson 精美打印示例

序列化`Employee`对象并漂亮地打印 JSON 输出的 Java 程序。

```java
public class Employee 
{
	private Integer id;
    private String firstName;
    private String lastName;
    private String email;

    //Constructors
    //Getters and setters
}

```

```java
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

public class Main 
{
	public static void main(String[] args) 
    {
		Employee employeeObj = new Employee(1, "Lokesh", "Gupta", "howtogoinjava@gmail.com");

		Gson gson = new GsonBuilder()
				.setPrettyPrinting()
				.create(); 

		System.out.println(gson.toJson(employeeObj));
    }
}

```

程序输出。

```java
{
  "id": 1,
  "firstName": "Lokesh",
  "lastName": "Gupta",
  "emailId": "howtogoinjava@gmail.com"
}

```

向我提供有关**的问题，以便在 Gson** 中实现精美打印。

学习愉快！

参考：

[Gson 用户指南](https://github.com/google/gson/blob/master/UserGuide.md)